﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
    
public partial class ViewStateTest : System.Web.UI.Page
{
    private bool executeFlag = true;
    
    protected void cmdSave_Click(object sender, EventArgs e)
    {
        // Сохраняем присланные значения текстовых полей
        SaveAllText(Table1.Controls, executeFlag);
    }
    
    private void SaveAllText(ControlCollection controls, bool saveNested)
    {
        if (!saveNested)
            return;
    
        // Перебираем все дочерние элементы в переданной коллекции таблицы 
        foreach (Control control in controls)
        {
            // Выбираем элементы текстовых полей и сохраняем
            // их содержимое в свойстве ViewState объекта страницы,
            // используя в качестве ключа идентификатор элемента
            if (control is TextBox)
            {
                this.ViewState[control.ID] = ((TextBox)control).Text;
            }
    
            // На верхних уровнях объекта Table1 дочерними элементами
            // являются строки и ячейки таблицы, поэтому продолжаем
            // рекурсивно спускаться ниже, пока не встретим объекты TextBox.
            // saveNested разрешает проверять вложенные уровни коллекций 
            if (control.Controls != null)
            {
                SaveAllText(control.Controls, saveNested);
            }
        }
    }
    
    protected void cmdRestore_Click(object sender, EventArgs e)
    {
        // Извлекаем пришедший с обратной отсылкой сохраненный текст предыдущей
        // обратной отсылки и вставляем его в текстовые поля, готовя новый отклик
        RestoreAllText(Table1.Controls, executeFlag);
    }
    
    void RestoreAllText(ControlCollection controls, bool restoreNested)
    {
        if (!restoreNested)
            return;
    
        // Перебираем все элементы таблицы, включая вложенные, чтобы
        // использовать их идентификаторы как ключи для извлечения
        // возможно сохраненных значений из словаря ViewState
        foreach (Control control in controls)
        {
            // Нашли текстовое поле 
            if (control is TextBox)
            {
                // Убеждаемся, что есть в словаре
                if (this.ViewState[control.ID] != null)
                {
                    // Приводим типы и восстанавливаем
                    ((TextBox)control).Text = (string)this.ViewState[control.ID];
                }
            }
    
            // Рекурсивно спускаемся ниже к дочерним элементам
            if (control.Controls != null)
            {
                RestoreAllText(control.Controls, restoreNested);
            }
        }
    }
}